#include "lwip/sys.h"

#define SYS_THREAD_MAX 4    //Эջ
u32_t lwip_sys_now;

u32_t sys_now(void)
{
  lwip_sys_now = xTaskGetTickCount();
  return lwip_sys_now;
}

int sys_sem_valid(sys_sem_t *sem)     //жһźǷЧ
{
  return (*sem != SYS_SEM_NULL);
}

void sys_sem_set_invalid(sys_sem_t *sem)  //һźΪЧ
{
  *sem = SYS_SEM_NULL;
}

int sys_mbox_valid(sys_mbox_t *mbox)    //жһǷЧ
{
  return (*mbox != SYS_MBOX_NULL);
}

void sys_mbox_set_invalid(sys_mbox_t *mbox)  //һΪЧ
{
  *mbox = SYS_MBOX_NULL;
}

void sys_init(void)   //ʼϵͳģ
{
    printf("[sys_arch] init ok\r\n");      //ӡʼɹϢ
}

err_t sys_sem_new(sys_sem_t *sem, u8_t count)   //½һź
{
  /*  sem */
  if(count <= 1)
  {
    *sem = xSemaphoreCreateBinary();
    if(count == 1)
    {
      sys_sem_signal(sem);
    }
  }
  else
    *sem = xSemaphoreCreateCounting(count,count);

#if SYS_STATS
    ++lwip_stats.sys.sem.used;
    if (lwip_stats.sys.sem.max < lwip_stats.sys.sem.used) {
        lwip_stats.sys.sem.max = lwip_stats.sys.sem.used;
    }
#endif /* SYS_STATS */

  if(*sem != SYS_SEM_NULL)
    return ERR_OK;
  else
  {
#if SYS_STATS
    ++lwip_stats.sys.sem.err;
#endif /* SYS_STATS */
    printf("[sys_arch]:new sem fail!\n");
    return ERR_MEM;
  }
}

void sys_sem_free(sys_sem_t *sem)   //ɾһź
{
#if SYS_STATS
   --lwip_stats.sys.sem.used;
#endif /* SYS_STATS */
  /* ɾ sem */
  vSemaphoreDelete(*sem);
  *sem = SYS_SEM_NULL;
}

void sys_sem_signal(sys_sem_t *sem)  //ͷһź
{
  if(xSemaphoreGive( *sem ) != pdTRUE)
    printf("[sys_arch]:sem signal fail!\n");
}

/*
 timeoutΪ㣬򷵻ֵΪ
 ȴźѵĺ
 źδָʱڷźţֵΪ
 SYS_ARCH_TIMEOUT̲߳صȴź
 ú㡣 */
u32_t sys_arch_sem_wait(sys_sem_t *sem, u32_t timeout)
{
  u32_t wait_tick = 0;
  u32_t start_tick = 0 ;

  //źǷЧ
  if(*sem == SYS_SEM_NULL)
    return SYS_ARCH_TIMEOUT;

  //Ȼȡʼȴźʱӽ
  start_tick = xTaskGetTickCount();

  //timeout != 0Ҫmsϵͳʱӽ
  if(timeout != 0)
  {
    //msתʱӽ
    wait_tick = timeout / portTICK_PERIOD_MS;
    if (wait_tick == 0)
      wait_tick = 1;
  }
  else
    wait_tick = portMAX_DELAY;  //һֱ

  //ȴɹȴʱ䣬ͱʾȴʱ
  if(xSemaphoreTake(*sem, wait_tick) == pdTRUE)
    return ((xTaskGetTickCount()-start_tick)*portTICK_RATE_MS);
  else
    return SYS_ARCH_TIMEOUT;
}

err_t sys_mbox_new(sys_mbox_t *mbox, int size)   //½һ
{
    /* Test_Queue */
  *mbox = xQueueCreate((UBaseType_t ) size,/* Ϣеĳ */
                       (UBaseType_t ) sizeof(void *));/* ϢĴС */
#if SYS_STATS
      ++lwip_stats.sys.mbox.used;
      if (lwip_stats.sys.mbox.max < lwip_stats.sys.mbox.used) {
         lwip_stats.sys.mbox.max = lwip_stats.sys.mbox.used;
      }
#endif /* SYS_STATS */
    if(NULL == *mbox)
    return ERR_MEM;

  return ERR_OK;
}

void sys_mbox_free(sys_mbox_t *mbox)  //ɾһ
{
  if( uxQueueMessagesWaiting( *mbox ) )
    {
        /* Line for breakpoint.  Should never break here! */
        portNOP();
#if SYS_STATS
        lwip_stats.sys.mbox.err++;
#endif /* SYS_STATS */

        // TODO notify the user of failure.
    }

  vQueueDelete(*mbox);

#if SYS_STATS
     --lwip_stats.sys.mbox.used;
#endif /* SYS_STATS */
}

void sys_mbox_post(sys_mbox_t *q, void *msg)  //ͶϢ
{
  while(xQueueSend( *q, /* Ϣеľ */
                    &msg,/* ͵Ϣ */
                    portMAX_DELAY) != pdTRUE); /* ȴʱ */
}

err_t sys_mbox_trypost(sys_mbox_t *q, void *msg)  //ͶϢ
{
  if(xQueueSend(*q,&msg,0) == pdPASS)
    return ERR_OK;
  else
    return ERR_MEM;
}

u32_t sys_arch_mbox_fetch(sys_mbox_t *q, void **msg, u32_t timeout)   //ȡϢ
{
  void *dummyptr;
  u32_t wait_tick = 0;
  u32_t start_tick = 0 ;

  if ( msg == NULL )  //洢ϢĵطǷЧ
        msg = &dummyptr;

  //Ȼȡʼȴźʱӽ
  start_tick = sys_now();

  //timeout != 0Ҫmsϵͳʱӽ
  if(timeout != 0)
  {
    //msתʱӽ
    wait_tick = timeout / portTICK_PERIOD_MS;
    if (wait_tick == 0)
      wait_tick = 1;
  }
  //һֱ
  else
    wait_tick = portMAX_DELAY;

  //ȴɹȴʱ䣬ͱʾȴʱ
  if(xQueueReceive(*q,&(*msg), wait_tick) == pdTRUE)
    return ((sys_now() - start_tick)*portTICK_PERIOD_MS);
  else
  {
    *msg = NULL;
    return SYS_ARCH_TIMEOUT;
  }
}

u32_t sys_arch_mbox_tryfetch(sys_mbox_t *q, void **msg)  //ԴȡϢ
{
    void *dummyptr;
    if ( msg == NULL )
        msg = &dummyptr;

  //ȴɹȴʱ
  if(xQueueReceive(*q,&(*msg), 0) == pdTRUE)
    return ERR_OK;
  else
    return SYS_MBOX_EMPTY;
}

sys_thread_t sys_thread_new(const char *name, lwip_thread_fn function, void *arg, int stacksize, int prio)  //
{
  sys_thread_t handle = NULL;
  BaseType_t xReturn = pdPASS;
  /* MidPriority_Task */
  xReturn = xTaskCreate((TaskFunction_t )function,  /* ں */
                        (const char*    )name,/*  */
                        (uint16_t       )stacksize,  /* ջС */
                        (void*          )arg,/* ں */
                        (UBaseType_t    )prio, /* ȼ */
                        (TaskHandle_t*  )&handle);/* ƿָ */
  if(xReturn != pdPASS)
  {
    printf("[sys_arch]:create task fail!err:%#lx\n",xReturn);
    return NULL;
  }
  return handle;
}

sys_prot_t sys_arch_protect(void)   //ٽ
{
    vPortEnterCritical();
    return 1;
}

void sys_arch_unprotect(sys_prot_t pval)   //˳ٽ
{
    ( void ) pval;
    vPortExitCritical();
}
